#!/usr/bin/env bash

API_SRC_PATH=src/AMR.elm

PERF_TIME_SRC_PATH=src/Time.elm
PERF_SPACE_SRC_PATH=src/Space.elm

EXAMPLE_SRC_PATH=src/Main.elm
EXAMPLE_OUT_PATH=main.js

EXAMPLE_MAKE_CMD="elm make $EXAMPLE_SRC_PATH --output=$EXAMPLE_OUT_PATH"

INVALID_ARG_MSG="Invalid argument '$1'.Try 'example', 'install', 'build', 'test', 'perf time' or 'perf space' instead!"
INVALID_INSTALL_ARG_MSG="Invalid argument '$2'.Try 'ci' or 'dev' instead!"
INVALID_PERF_ARG_MSG="Invalid argument for perf '$2'.Try 'space' or 'time' instead!"

PORT_NO=8000

function main {
    case $1 in
        install)
            case $2 in
                ci) ci
                ;;
                dev) dev
                ;;
                *) echo $INVALID_INSTALL_ARG_MSG
                ;;
            esac
        ;;
        perf)
            case $2 in
                time) time
                ;;
                space) space
                ;;
                *) echo $INVALID_PERF_ARG_MSG
                ;;
            esac
        ;;
        example) example
        ;;
        build) build
        ;;
        test) test
        ;;
        *) echo $INVALID_ARG_MSG
        ;;
    esac
}

function example {
    cd example
    eval $EXAMPLE_MAKE_CMD
    live-server --no-css-inject --port=$PORT_NO --watch=$EXAMPLE_OUT_PATH & nodemon --watch ../ --ext elm --exec $EXAMPLE_MAKE_CMD
}

function ci {
    npm install -g elm@0.19.0-bugfix6
    npm install -g elm-test@0.19.0-rev6
}

function dev {
    npm install -g live-server@1.2.1
    npm install -g nodemon@1.19.1
    npm install -g elm-format@0.8.1
    ci  
}

function build {
    echo "Compiling package..."
    elm make $API_SRC_PATH
    
    cd example
    echo "Compiling example..."
    elm make --output=/dev/null $EXAMPLE_SRC_PATH
    
    cd ../perf
    echo "Compiling time performance benchmarks..."
    elm make --output=/dev/null $PERF_TIME_SRC_PATH
    echo "Compiling space performance checks..."
    elm make --output=/dev/null $PERF_SPACE_SRC_PATH
}

function test {
    echo "Running tests..."
    elm-test
}

function time {
    cd perf
    sleep 1 && open "http://localhost:$PORT_NO/$PERF_TIME_SRC_PATH" & elm reactor
}

function space {
    cd perf
    sleep 1 && open "http://localhost:$PORT_NO/$PERF_SPACE_SRC_PATH" & elm reactor            
}

main $1 $2